@@ -54,13 +54,14 @@ const nonSettings = [
 
 // This is a function to make it easy to create a new instance. It is important to not reuse a
 // config object after passing it to log4js.configure() because that method mutates the object. :(
-const defaultLogConfig = (level: string) => ({
-    appenders: {console: {type: 'console'}},
+const defaultLogConfig = (level: string, layoutType: string) => ({
+    appenders: {console: {type: 'console', layout: {type: layoutType}}},
     categories: {
         default: {appenders: ['console'], level},
     }
 });
 const defaultLogLevel = 'INFO';
+const defaultLogLayoutType = 'colored';
 
 const initLogging = (config: any) => {
     // log4js.configure() modifies exports.logconfig so check for equality first.
@@ -76,7 +77,7 @@ const initLogging = (config: any) => {
 
 // Initialize logging as early as possible with reasonable defaults. Logging will be re-initialized
 // with the user's chosen log level and logger config after the settings have been loaded.
-initLogging(defaultLogConfig(defaultLogLevel));
+initLogging(defaultLogConfig(defaultLogLevel, defaultLogLayoutType));
 
 /* Root path of the installation */
 exports.root = absolutePaths.findEtherpadRoot();
@@ -106,6 +107,7 @@ exports.ttl = {
     RefreshToken: 1 * 24 * 60 * 60, // 1 day in seconds
 }
 
+exports.updateServer = "https://static.etherpad.org"
 
 
 /*
@@ -168,11 +170,11 @@ exports.authenticationMethod = 'sso'
 /*
  * The Type of the database
  */
-exports.dbType = 'dirty';
+exports.dbType = 'rustydb';
 /**
  * This setting is passed with dbType to ueberDB to set up the database
  */
-exports.dbSettings = {filename: path.join(exports.root, 'var/dirty.db')};
+exports.dbSettings = {filename: path.join(exports.root, 'var/rusty.db')};
 
 /**
  * The default Text of a new pad
@@ -291,6 +293,11 @@ exports.allowUnknownFileEnds = true;
  */
 exports.loglevel = defaultLogLevel;
 
+/**
+ * The log layout type of log4js
+ */
+exports.logLayoutType = defaultLogLayoutType;
+
 /**
  * Disable IP logging
  */
@@ -373,6 +380,14 @@ exports.sso = {
  */
 exports.showSettingsInAdminPage = true;
 
+/*
+ * Settings for cleanup of pads
+ */
+exports.cleanup = {
+  enabled: false,
+  keepRevisions: 100,
+}
+
 /*
  * By default, when caret is moved out of viewport, it scrolls the minimum
  * height needed to make this line visible.
@@ -817,7 +832,12 @@ exports.reloadSettings = () => {
     storeSettings(credentials);
 
     // Init logging config
-    exports.logconfig = defaultLogConfig(exports.loglevel ? exports.loglevel : defaultLogLevel);
+    exports.logconfig = defaultLogConfig(
+        exports.loglevel ? exports.loglevel : defaultLogLevel,
+        exports.logLayoutType ? exports.logLayoutType : defaultLogLayoutType
+    );
+    logger.warn("loglevel: " + exports.loglevel);
+    logger.warn("logLayoutType: " + exports.logLayoutType);
     initLogging(exports.logconfig);
 
     if (!exports.skinName) {
@@ -826,6 +846,11 @@ exports.reloadSettings = () => {
         exports.skinName = 'colibris';
     }
 
+    if (!exports.socketTransportProtocols.includes("websocket") || !exports.socketTransportProtocols.includes("polling")) {
+        logger.warn("Invalid socketTransportProtocols setting. Please check out settings.json.template and update your settings.json. Falling back to the default ['websocket', 'polling'].");
+        exports.socketTransportProtocols = ['websocket', 'polling'];
+    }
+
     // checks if skinName has an acceptable value, otherwise falls back to "colibris"
     if (exports.skinName) {
         const skinBasePath = path.join(exports.root, 'src', 'static', 'skins');
@@ -925,6 +950,12 @@ exports.reloadSettings = () => {
         logger.warn(`${dirtyWarning} File location: ${exports.dbSettings.filename}`);
     }
 
+    if (exports.dbType === 'rustydb' || exports.dbType === "sqlite") {
+      exports.dbSettings.filename = absolutePaths.makeAbsolute(exports.dbSettings.filename);
+      logger.warn(`File location: ${exports.dbSettings.filename}`);
+    }
+
+
     if (exports.ip === '') {
         // using Unix socket for connectivity
         logger.warn('The settings file contains an empty string ("") for the "ip" parameter. The ' +
